<?php

declare(strict_types = 1);

namespace Drupal\ui_skins\HookHandler;

use Drupal\Component\Utility\NestedArray;
use Drupal\ui_skins\UiSkinsUtility;

/**
 * Inject inline CSS.
 */
class PageTop {

  /**
   * The page top key for CSS variables.
   */
  public const PAGE_TOP_CSS_VARIABLES_KEY = 'ui_skins_css_variables';

  /**
   * Inject inline CSS variables.
   *
   * @param array $page_top
   *   The page top elements.
   */
  public function alter(array &$page_top): void {
    $ui_skins_css_variables_settings = \theme_get_setting(FormSystemThemeSettingsAlter::CSS_VARIABLES_THEME_SETTING_KEY);
    if (!\is_array($ui_skins_css_variables_settings)) {
      return;
    }

    // Prepare list of variables grouped by scope.
    $css_variables = [];
    foreach ($ui_skins_css_variables_settings as $plugin_id => $scoped_values) {
      foreach ($scoped_values as $scope => $value) {
        $css_variables = NestedArray::mergeDeep($css_variables, [
          UiSkinsUtility::getCssScopeName($scope) => [
            UiSkinsUtility::getCssVariableName($plugin_id) => $value,
          ],
        ]);
      }
    }

    if (empty($css_variables)) {
      return;
    }

    $page_top[static::PAGE_TOP_CSS_VARIABLES_KEY] = [
      '#type' => 'html_tag',
      '#tag' => 'style',
      '#value' => UiSkinsUtility::getCssVariablesInlineCss($css_variables),
    ];
  }

}
